﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ADSMSLongSmpl
{
    public partial class Form1 : Form
    {
        private int SaveLeng;//ファイルへの保存制限数(0の時制限なし)
        private StreamWriter SaveFile;//保存ファイルストリーム
        private adsms_setting smpl_setting;//取り込み設定
        private int TotalLeng;//取り込み済みデータ長
        private Boolean StopReq;//サンプリング停止要求
        private short[,] SamplingBuffer;//取り込みバッファ

        //チャンネル選択チェックボックス配列処理用
        private System.Windows.Forms.CheckBox[] ChChkArray;
        //レンジ選択コンボボックス配列処理用
        private System.Windows.Forms.ComboBox[] RangeSelArray;

        public Form1()
        {
            InitializeComponent();
        }

        private void SettingInit()
        {
            int ch;

            //ID選択コンボボックスを初期化します
            IdSel.SelectedIndex = 0;
            //コントロールの配列に画面のコントロールを設定します
            ChChkArray = new System.Windows.Forms.CheckBox[] { ChChk1, ChChk2, ChChk3, ChChk4,
                                                          ChChk5, ChChk6, ChChk7, ChChk8,
                                                          ChChk9, ChChk10, ChChk11, ChChk12,
                                                          ChChk13, ChChk14, ChChk15, ChChk16};
            RangeSelArray = new System.Windows.Forms.ComboBox[] {RangeSel1, RangeSel2, RangeSel3, RangeSel4,
                                                             RangeSel5, RangeSel6, RangeSel7, RangeSel8,
                                                             RangeSel9, RangeSel10, RangeSel11, RangeSel12,
                                                             RangeSel13, RangeSel14, RangeSel15, RangeSel16};

            for (ch = 0; ch < 16; ch++)
            {
                RangeSelArray[ch].SelectedIndex = 0;
            }

            //トリガ選択コンボボックスを初期化
            TriggerTypeSel.SelectedIndex = 0;

            //トリガチャンネル選択コンボボックスを初期化
            TriggerCh.SelectedIndex = 0;
            //その他初期化
            TriggerLevel.Text = "2048"; //トリガレベルを中間値
            ClkSel1.Checked = true; //内部クロック有効
            ClockInterval.Text = "1000"; //内部クロックを1kHz
            PreTriggerLen.Text = "0"; //プレトリガ長を0
            LimitLen.Text = "1000000"; //取り込み制限長1000000
            StatusView.Text = "デバイスがオープンされていません"; //ステータス表示
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SettingInit();
            //イネーブルディセーブル制御
            EnableDisableControl(0);
        }

        //イネーブルディセーブル制御(画面のコントロール表示切替)
        private void EnableDisableControl(int sel)
        {
            int ch;

            switch( sel)
            {
            case 0://オープンしていない
                OpenBtn.Enabled = true;
                CloseBtn.Enabled = false;
                StartBtn.Enabled = false;
                StopBtn.Enabled = false;
                IdSel.Enabled = true;
                TriggerBtn.Enabled = false;

                for(ch = 0 ; ch < 16 ; ch++ )
                {
                    ChChkArray[ch].Enabled = false;
                    RangeSelArray[ch].Enabled = false;
                }

                TriggerTypeSel.Enabled = false;
                TriggerLevel.Enabled = false;
                TriggerCh.Enabled = false;
                ClkSel1.Enabled = false;
                ClkSel2.Enabled = false;
                ClockInterval.Enabled = false;
                PreTriggerLen.Enabled = false;
                LimitLen.Enabled = false;
                break;
            case 1://オープンしている(待機)
                OpenBtn.Enabled = false;
                CloseBtn.Enabled = true;
                StartBtn.Enabled = true;
                StopBtn.Enabled = false;
                IdSel.Enabled = false;
                TriggerBtn.Enabled = false;

                for(ch = 0 ; ch < 16 ; ch++ )
                {
                    ChChkArray[ch].Enabled = true;
                    RangeSelArray[ch].Enabled = true;
                }

                TriggerTypeSel.Enabled = true;
                TriggerLevel.Enabled = true;
                TriggerCh.Enabled = true;
                ClkSel1.Enabled = true;
                ClkSel2.Enabled = true;
                ClockInterval.Enabled = true;
                PreTriggerLen.Enabled = true;
                LimitLen.Enabled = true;
                break;
            case 2://オープンしている(サンプリング中)
                OpenBtn.Enabled = false;
                CloseBtn.Enabled = false;
                StartBtn.Enabled = false;
                StopBtn.Enabled = true;
                IdSel.Enabled = false;
                if( TriggerTypeSel.SelectedIndex == 0 )
                    TriggerBtn.Enabled = true;
                else
                    TriggerBtn.Enabled = false;

                for(ch = 0 ; ch < 16 ; ch++ )
                {
                    ChChkArray[ch].Enabled = false;
                    RangeSelArray[ch].Enabled = false;
                }

                TriggerTypeSel.Enabled = false;
                TriggerLevel.Enabled = false;
                TriggerCh.Enabled = false;
                ClkSel1.Enabled = false;
                ClkSel2.Enabled = false;
                ClockInterval.Enabled = false;
                PreTriggerLen.Enabled = false;
                LimitLen.Enabled = false;
                break;
            }
        }

        //デバイスオープンボタン
        private void OpenBtn_Click(object sender, EventArgs e)
        {
            short ret;

            ret = USBADSMS.Tusbadsms_Device_Open((short)IdSel.SelectedIndex);

            if( ret != 0 )//エラー発生
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
            else
            {
                //イネーブルディセーブル制御
                EnableDisableControl(1);
                StatusView.Text = "オープンされました";
            }
        }

        //デバイスクローズボタン
        private void CloseBtn_Click(object sender, EventArgs e)
        {
            USBADSMS.Tusbadsms_Device_Close((short)IdSel.SelectedIndex);
            //イネーブルディセーブル制御
            EnableDisableControl(0);
            StatusView.Text = "デバイスがオープンされていません";
        }

        //終了処理
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (CloseBtn.Enabled)//クローズしないで終了した場合は
            {
                if (StopBtn.Enabled)//サンプリングがストップしていない場合は
                {
                    StopReq = true; //サンプリング終了
                }
                USBADSMS.Tusbadsms_Device_Close((short)IdSel.SelectedIndex);
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            short ret;
            byte status;
            byte ovf;
            int leng;
            int buflen;
            short[] tmpData;

            int DSize;

            if (StopReq)//停止要求の確認
            {
                SampleStop();
                StatusView.Text = "サンプリング終了\r\n" + "データ数 = " + TotalLeng.ToString("0");
                return;
            }

            //バッファステータスの確認
            ret = USBADSMS.Tusbadsms_Status_Read((short)IdSel.SelectedIndex, out status, out ovf, out leng);

            if (ret != 0)
            {
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
                SampleStop();
            }

            switch( status)
            {
            case 0://停止中
                StatusView.Text = "停止中です";
                Timer1.Enabled = true;
                break;
            case 1://トリガ待ち
                StatusView.Text = "トリガ待ちです";
                Timer1.Enabled = true;
                break;
            case 2://サンプリング中
                if( ovf != 0 )//オーバーフローの時
                {
                    MessageBox.Show("バッファオーバーフローです");
                    SampleStop();
                    return;
                }

                if( ( SaveLeng != 0 ) & ((TotalLeng + leng) >= SaveLeng) )
                {
                    leng = SaveLeng - TotalLeng;
                    buflen = leng;

                    tmpData = new short[buflen * smpl_setting.ChLen];//データ処理用バッファの確保

                    DSize = tmpData.Length * 2;

                    ret = USBADSMS.Tusbadsms_Memory_Read((short) IdSel.SelectedIndex,tmpData,ref leng);//データの読み込み

                    CopyData(tmpData, leng);
                    TotalLeng += leng;
                    SampleStop();
                    StatusView.Text = "サンプリング終了\r\n" + "データ数 = " + TotalLeng.ToString("0");
                }
                else
                {
                    buflen = leng;
                    tmpData = new short[buflen * smpl_setting.ChLen];//データ処理用バッファの確保

                    DSize = tmpData.Length * 2;

                    ret = USBADSMS.Tusbadsms_Memory_Read((short) IdSel.SelectedIndex, tmpData,ref leng);//データの読み込み

                    CopyData(tmpData, leng);
                    TotalLeng += leng;
                    StatusView.Text = "サンプリング中です\r\n" + "データ数 = " + TotalLeng.ToString("0");
                    Timer1.Enabled = true;
                }
                break;
            }
        }

        //データのコピー処理
        private void CopyData(short[] Data, long leng)
        {
            long pointer;
            long ch;
            long cnt;

            pointer = 0;
            for (ch = 0; ch < smpl_setting.ChLen; ch++)
            {
                for (cnt = TotalLeng; cnt < (TotalLeng + leng); cnt++ )
                {
                    SamplingBuffer[ch,cnt] = Data[pointer];
                    pointer += 1;
                }
            }
        }

        //データの書き込み処理
        private void WriteData()
        {
            long cnt;
            long ch;
            for( cnt = 0 ; cnt < TotalLeng ; cnt++ )
            {
                for( ch = 0 ; ch < smpl_setting.ChLen ; ch++ )
                {
                    if (ch == ( smpl_setting.ChLen - 1))
                        SaveFile.WriteLine(SamplingBuffer[ch, cnt].ToString("0"));
                    else
                        SaveFile.Write(SamplingBuffer[ch, cnt].ToString("0") + ",");
                }
            }
        }

        //連続取り込み停止時の処理
        private void SampleStop()
        {
            Timer1.Enabled = false;
            USBADSMS.Tusbadsms_Sampling_Stop((short) IdSel.SelectedIndex);//サンプリング停止
            StatusView.Text = "データ保存中";
            Refresh();
            WriteData();
            SaveFile.Close();//ファイルを閉じる
            EnableDisableControl(1);
        }

        private void StartBtn_Click(object sender, EventArgs e)
        {
            short ret;
            int ch;

            //チャンネル
            smpl_setting.ChLen = 0;
            if (smpl_setting.ChPattern == null)
                smpl_setting.ChPattern = new byte[16];
            if (smpl_setting.ChPatternRange == null)
                smpl_setting.ChPatternRange = new byte[16];
            for (ch = 0; ch < 16; ch++)
            {
                if (ChChkArray[ch].Checked)
                {
                    smpl_setting.ChPattern[smpl_setting.ChLen] = (byte) ch;
                    smpl_setting.ChPatternRange[smpl_setting.ChLen] = (byte) RangeSelArray[ch].SelectedIndex;
                    smpl_setting.ChLen += 1;
                }
            }
            if (smpl_setting.ChLen == 0)
            {
                MessageBox.Show("チャンネルが選択されていません");
                return;
            }

            //トリガ設定
            smpl_setting.TriggerType = (byte)(TriggerTypeSel.SelectedIndex + 1);
            if (smpl_setting.TriggerType >= 3)
            {
                if (short.TryParse(TriggerLevel.Text, out smpl_setting.TriggerLevel))
                {
                    if (smpl_setting.TriggerLevel < 1 || smpl_setting.TriggerLevel > 4094)
                    {
                        MessageBox.Show("トリガレベルが不正です");
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("トリガレベル入力値が不正です");
                    return;
                }
            }

            //トリガチャンネル

            smpl_setting.TriggerCh = (byte)TriggerCh.SelectedIndex;

            //クロック設定
            if (ClkSel1.Checked)
            {
                smpl_setting.ClockSel = 0;
                if (int.TryParse(ClockInterval.Text, out smpl_setting.SamplingClock))
                {
                    if (smpl_setting.SamplingClock > 16777215 || smpl_setting.SamplingClock < 10)
                    {
                        MessageBox.Show("クロック間隔設定値が不正です");
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("クロック間隔入力値が不正です");
                    return;
                }
            }
            else
                smpl_setting.ClockSel = 1;

            //プレトリガ長
            if (int.TryParse(PreTriggerLen.Text, out smpl_setting.PreTriggerLen))
            {
                if (smpl_setting.PreTriggerLen > 10000000 || smpl_setting.PreTriggerLen < 0)
                {
                    MessageBox.Show("プレトリガ設定値が不正です");
                    return;
                }
            }
            else
            {
                MessageBox.Show("プレトリガ入力値が不正です");
                return;
            }

            //PC内　データ取込バッファ
            smpl_setting.TotalLen = 1000000;

            //ファイルへの最大保存数
            if( int.TryParse(LimitLen.Text,out SaveLeng ) )
            {
                if( SaveLeng < 0 || SaveLeng > 100000000 )
                {
                    MessageBox.Show("保存数の設定値が不正です");
                    return;
                }
                else
                {
                    SamplingBuffer = new short[smpl_setting.ChLen,SaveLeng];
                }
            }
            else
            {
                MessageBox.Show("保存数の入力値が不正です");
                return;
            }

            //保存先ファイル名の取得
            SaveFileDialog1.OverwritePrompt = true;
            SaveFileDialog1.Filter = "データファイル(*.dat)|*.dat";
            if( SaveFileDialog1.ShowDialog() == DialogResult.OK )
            {
                if( System.IO.File.Exists(SaveFileDialog1.FileName))//既存ファイルチェック
                {
                    System.IO.File.Delete(SaveFileDialog1.FileName);
                }

                SaveFile = new StreamWriter(File.OpenWrite(SaveFileDialog1.FileName));//ファイルを開く
                ret = USBADSMS.Tusbadsms_Sample_Start((short) IdSel.SelectedIndex,ref smpl_setting);
                if( ret == 0 )//正常にスタートできた
                {
                    StopReq = false;
                    TotalLeng = 0;
                    EnableDisableControl(2);
                    Timer1.Interval = 100;
                    Timer1.Enabled = true;
                }
                else//正常にスタートできなかった
                {
                    SaveFile.Close();
                    USBADSMS.Tusbadsms_Device_Close((short) IdSel.SelectedIndex);
                }
            }
        }

        private void TriggerBtn_Click(object sender, EventArgs e)
        {
            short ret;

            ret = USBADSMS.Tusbadsms_Sampling_Trigger((short) IdSel.SelectedIndex);//トリガコマンド
            if (ret != 0)
            {
                USBADSMS.Tusbadsms_Device_Close((short)IdSel.SelectedIndex);
                StopReq = true;
            }
        }

        private void StopBtn_Click(object sender, EventArgs e)
        {
            StopReq = true;
        }
    }
}
