﻿//-----------------------------------------------
//TUSB-1612ADSM-S用ドライバアクセス用API
//Visual C# 用モジュール
//
//2013_06_28　株式会社タートル工業
//Copyright (C) 2013 Turtle Industry Co.,Ltd.
//-----------------------------------------------

using System.Runtime.InteropServices;

// 連続取り込み設定用構造体宣言
[StructLayout(LayoutKind.Sequential)] struct adsms_setting
{
    public byte ChLen; //チャンネル長 1-16
    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
    public byte[] ChPattern;
    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
    public byte[] ChPatternRange;
    public byte TriggerType; //トリガ種類　1:ソフト　2:外部ディジタル立上り 3:外部ディジタル立下り 4:立上エッジ 5:立下エッジ　6:上限レベル　7:下限レベル
    public short TriggerLevel; //エッジやレベルの閾値
    public byte TriggerCh; //エッジやレベルを検出するチャンネル位置(パターンバッファのポインタ)
    public byte ClockSel; //0:内部クロック　1:外部クロック
    public int SamplingClock; //内部クロック間隔 10～16,777,215[uS]
    public int PreTriggerLen; //プレトリガ長 0～10,000,000
    public int TotalLen; //全取込データバッファ 1～10,000,000
}

class USBADSMS
{
    //API関数宣言

    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Device_Open(short id);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern void Tusbadsms_Device_Close(short id);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Pio_Write(short id, byte dat);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Pio_Read(short id, out byte dat);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Single_Sample(short id, byte ch, byte range, out short dat);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Status_Read(short id, out byte status, out byte ovf,out int leng);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Memory_Read(short id, [Out] short[] Data,ref int leng);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Sampling_Stop(short id);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Memory_Clear(short id);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Sampling_Trigger(short id);
    [DllImport("TUSADSMS.DLL", CallingConvention = CallingConvention.Cdecl)]
    public static extern short Tusbadsms_Sample_Start(short id, ref adsms_setting smplset);

    public static string GetErrMessage(short retcode)
    {
        switch (retcode)
        {
            case 0:
                return "正常終了しました";
            case 1:
                return "ID番号が不正です";
            case 2:
                return "ドライバがインストールされていません";
            case 3:
                return "すでにデバイスはオープンされています";
            case 4:
                return "接続されている台数が多すぎます";
            case 5:
                return "オープンできませんでした";
            case 6:
                return "デバイスがみつかりません";
            case 7:
                return "オープンされていません";
            case 8:
                return "パラメータエラー";
            case 9:
                return "USB通信エラーです";
            default:
                return "不明なエラーです";
        }
    }
}
