﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ADSMSSimple
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int id;
            int ch;

            //ID選択コンボボックスを初期化します
            for( id = 0 ; id < 16 ; id++ )
            {
                IdSel.Items.Add(id.ToString("00"));
            }
            IdSel.SelectedIndex = 0;

            //レンジ選択コンボボックスを初期化します
            RangeSel.Items.Add("±10V");
            RangeSel.Items.Add("±5V");
            RangeSel.Items.Add("±2.5V");
            RangeSel.Items.Add("±1V");
            RangeSel.Items.Add("±0.5V");
            RangeSel.Items.Add("±0.25V");
            RangeSel.Items.Add("±0.1V");
            RangeSel.Items.Add("10V");
            RangeSel.Items.Add("5V");
            RangeSel.Items.Add("2.5V");
            RangeSel.Items.Add("1V");
            RangeSel.Items.Add("0.5V");
            RangeSel.Items.Add("0.25V");
            RangeSel.Items.Add("0.1V");
            RangeSel.SelectedIndex = 0;

            //チャンネル選択コンボボックスを初期化します
            for( ch = 0 ; ch < 16 ; ch++ )
                ChSel.Items.Add(ch.ToString("00"));
            ChSel.SelectedIndex = 0;

            //その他初期化
            DataDisp.Text = "";
            RawDisp.Text = "";

            //イネーブルディセール制御
            EnableDisableControl(false);
        }

        //イネーブルディセーブル制御
        private void EnableDisableControl(bool sel)
        {
            DOBtn.Enabled = sel;
            DIBtn.Enabled = sel;
            SingleSampleBtn.Enabled = sel;
            DO0Chk.Enabled = sel;
            DO1Chk.Enabled = sel;
            RangeSel.Enabled = sel;
            ChSel.Enabled = sel;
            IdSel.Enabled = !sel;
            OpenBtn.Enabled = !sel;
            CloseBtn.Enabled = sel;
        }

        private void OpenBtn_Click(object sender, EventArgs e)
        {
            short ret;

            ret = USBADSMS.Tusbadsms_Device_Open((short) IdSel.SelectedIndex);
            if (ret != 0)//エラー発生
            {
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
            }
            else//成功
            {
                //イネーブルディセーブル制御
                EnableDisableControl(true);
            }
        }

        private void CloseBtn_Click(object sender, EventArgs e)
        {
            USBADSMS.Tusbadsms_Device_Close((short)IdSel.SelectedIndex);
            //イネーブルディセーブル制御
            EnableDisableControl(false);
        }

        private void DOBtn_Click(object sender, EventArgs e)
        {
            short ret;
            byte dat;

            //ビットパターン作成
            dat  = 0;
            if( DO0Chk.Checked )
                dat += (byte) 1;
            if( DO1Chk.Checked )
                dat += (byte)2;

            //出力
            ret = USBADSMS.Tusbadsms_Pio_Write((short) IdSel.SelectedIndex, dat);

            //エラーチェック
            if( ret != 0 )
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
        }

        private void DIBtn_Click(object sender, EventArgs e)
        {
            short ret;
            byte dat;

            //入力
            ret = USBADSMS.Tusbadsms_Pio_Read((short) IdSel.SelectedIndex, out dat);

            //エラーチェック
            if( ret != 0 )
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
            else
            {
                if( (dat & 1 ) != 0 )//ビット0がHIGHならば
                    DI0Lmp.Image = ImageList1.Images[1];
                else//LOWならば
                    DI0Lmp.Image = ImageList1.Images[0];

                if( (dat & 2 ) != 0 )//ビット1がHIGHならば
                    DI1Lmp.Image = ImageList1.Images[1];
                else//LOWならば
                    DI1Lmp.Image = ImageList1.Images[0];
            }
        }

        private void SingleSampleBtn_Click(object sender, EventArgs e)
        {
            short ret;
            short dat;

            ret = USBADSMS.Tusbadsms_Single_Sample((short)IdSel.SelectedIndex, (byte) ChSel.SelectedIndex, (byte) RangeSel.SelectedIndex, out dat);

            if( ret != 0 )
                MessageBox.Show(USBADSMS.GetErrMessage(ret));
            else
            {
                DataDisp.Text = dat.ToString("0000");
                RawDisp.Text = RawToVolt(dat, (byte) RangeSel.SelectedIndex).ToString("0.0000");
            }
        }
        private double RawToVolt(short dat, byte range)
        {
            switch(range)
            {
                case 0://±10Vレンジ
                    return (double) dat / 4096.0 * 20.0 - 10.0;
                case 1://±5Vレンジ
                    return (double) dat / 4096.0 * 10.0 - 5.0;
                case 2://±2.5Vレンジ
                    return (double) dat / 4096.0 * 5.0 - 2.5;
                case 3://±1Vレンジ
                    return (double) dat / 4096.0 * 2.0 - 1.0;
                case 4://±0.5Vレンジ
                    return (double) dat / 4096.0 * 1.0 - 0.5;
                case 5://±0.25Vレンジ
                    return (double) dat / 4096.0 * 0.5 - 0.25;
                case 6://±0.1Vレンジ
                    return (double) dat / 4096.0 * 0.2 - 0.1;
                case 7://10Vレンジ
                    return (double) dat / 4096.0 * 10.0;
                case 8://5Vレンジ
                    return (double) dat / 4096.0 * 5.0;
                case 9://2.5Vレンジ
                    return (double) dat / 4096.0 * 2.5;
                case 10://1Vレンジ
                    return (double) dat / 4096.0 * 1.0;
                case 11://0.5Vレンジ
                    return (double) dat / 4096.0 * 0.5;
                case 12://0.25Vレンジ
                    return (double) dat / 4096.0 * 0.25;
                case 13://0.1Vレンジ
                    return (double) dat / 4096.0 * 0.1;
                default:
                    return 0.0;
            }
        }
    }
}
